// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file FileWriter.h */

// Marcel Schiffel, 23.06.09

#ifndef AMDIS_POVRAYWRITER_H
#define AMDIS_POVRAYWRITER_H

#include <string>
#include <fstream>
#include <iostream>
#include "DataCollector.h"
#include "FileWriter.h"

using namespace std;

namespace AMDiS {


  typedef struct {
    double minx;
    double maxx;
    double miny;
    double maxy;
    double minz;
    double maxz;
  } BoundingBox;


  class PovrayWriter {
  public:
    PovrayWriter(FileWriter *fw, DataCollector<> *dc) : fileWriter(fw), dataCollector(dc)
    {
//       FUNCNAME("PovrayWriter::PovrayWriter()");	

      // abort if problem not 3D //FIX THAT!!!
      /*	TEST_EXIT(dataCollector->getMesh()->getDim()==3)
		("Povray export only possible for 3D data!");*/  

      bBox = NULL;
    }

    ~PovrayWriter();
    int writeFile(const std::string filename);

    // provides the bounding box of the mesh (lazy evaluation, bounding box is computed only once)
    BoundingBox *getBoundingBox(ofstream&);

  private:
    FileWriter *fileWriter;
    DataCollector<> *dataCollector;
    BoundingBox *bBox;

    // TODO: remove!
    void tryMeshTraversal(ofstream&);

    void writeTestStuff(ofstream&, DataCollector<>&); // TODO: remove/rename
    void writeHeader(ofstream&);
    void writeIncludes(ofstream&);
    void writeCamera(ofstream&);
    void writeLight(ofstream&);
    void writeMesh2(ofstream&, DataCollector<>&);
    void writeVertexVectors(ofstream&, DataCollector<>&);
    void writeTextureList(ofstream&, DataCollector<>&);
    void writeFaceIndices(ofstream&, DataCollector<>&);
  };

} // end of namespace AMDiS

#endif
