// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file PetscSolverCahnHilliard2.h */

#ifndef AMDIS_PETSC_SOLVER_CAHN_HILLIARD2_H
#define AMDIS_PETSC_SOLVER_CAHN_HILLIARD2_H

#include "parallel/PetscSolverGlobalMatrix.h"

namespace AMDiS {

  using namespace std;

  struct CahnHilliardData2 {
    KSP kspMass, kspMinusDeltaK, kspMplusK;
    Mat matMass, matMinusDeltaK;
    double *eps, *delta;
    MPI::Intracomm *mpiCommGlobal;
  };

 class PetscSolverCahnHilliard2 : public PetscSolverGlobalMatrix
  {
  public:
    /// Creator class
    class Creator : public OEMSolverCreator
    {
    public:
      virtual ~Creator() {}

      /// Returns a new PetscSolver object.
      OEMSolver* create() 
      { 
	return new PetscSolverCahnHilliard2(this->name); 
      }
    };

    PetscSolverCahnHilliard2(string name);

    void solvePetscMatrix(SystemVector &vec, AdaptInfo *adaptInfo);
  
    void setChData(double *epsPtr, double *deltaPtr)
    {
      eps = epsPtr;
      delta = deltaPtr;
    }
    
    
    void setPhase(DOFVector<double> *d, double eP3=0)
    {
      phase = d;
      epsPhase3 = eP3;
    }

  protected:
    void initSolver(KSP &ksp);

    void initPreconditioner(PC pc);

    void exitPreconditioner(PC pc);

  private:
    int pressureComponent;

    bool pressureNullSpace;

    /// If true, old solution is used for initial guess in solver phase.
    bool useOldInitialGuess;

    /// 0: approximate solve   1: direct solver
    int velocitySolutionMode;

    /// 0: approximate solve   1: direct solver
    int massSolutionMode;

    /// 0: approximate solve   1: direct solver
    int laplaceSolutionMode;

    PetscSolver *massMatrixSolver, *laplaceMatrixSolver, *deltaKMatrixSolver;

    CahnHilliardData2 matShellContext;

    double *eps, *delta;
    
    
    double epsPhase3;

    SystemVector* solution;

    DOFVector<double>* phase;

  };

}

#endif
