#include "ElementFunction.h"

namespace experimental {

  template<>
  void ElementBox<PointType_, std::pair<ElInfoDataType, PointType_> >::fillBox(const FiniteElemSpace* feSpace)
  {
    TraverseStack stack;
    ElInfo *elInfo = stack.traverseFirst(feSpace->getMesh(), -1, Mesh::CALL_LEAF_EL | Mesh::FILL_COORDS);

    int dim = feSpace->getMesh()->getDim();
    while (elInfo) {
      ElInfo* macroElInfo = feSpace->getMesh()->createNewElInfo();
      macroElInfo->fillMacroInfo(elInfo->getMacroElement());
      stack.fillRefinementPath(*elInfo, *macroElInfo);

      ElInfoDataType elInfoData = boost::tuples::make_tuple(elInfo->getMacroElement(), elInfo->getRefinementPathLength(), elInfo->getRefinementPath());

      WorldVector<double> centroid = elInfo->getCoord(0);
      for (int i = 1; i < dim+1; i++)
	centroid += elInfo->getCoord(i);
      centroid *= 1.0/(dim+1.0);

      addData(centroid, std::make_pair(elInfoData, centroid));

      elInfo = stack.traverseNext(elInfo);
    }
    boxFilled = true;
  }
  

  template<>
  bool ElementBox<PointType_, DataType_>::getNearestElInfo(PointType_ &x, ElInfo*& minElInfo)
  {
    DataType_ data;
    if (getNearestData(x, data)) {
      minElInfo->fillElInfo(data.first.get<0>(), data.first.get<1>(), data.first.get<2>());
      return true;
    } else
      return false;
  }
  

  template<typename PointType, typename DataType>
  template<typename T>
  T ElementBox<PointType,DataType>::evalAtPoint_simple(DOFVector<T>& vec, PointType& x)
  {
    ElInfo* elInfo = vec.getFeSpace()->getMesh()->createNewElInfo();
    elInfo->setFillFlag(Mesh::FILL_COORDS);
    T value; nullify(value);
    if (getNearestElInfo(x, elInfo)) {
      std::cout<<"elInfo = "<<elInfo->getElement()->getIndex()<<"\n";
      int dim = vec.getFeSpace()->getMesh()->getDim();
      DimVec<double> lambda(dim, NO_INIT);
      for (int i = 0; i < dim+1; i++)
	lambda[i] = 1.0/std::max(1.e-8, norm(x - elInfo->getCoord(i)));
      // normalize barycentric coords
      double sum = 0.0;
      for (int i = 0; i < dim+1; i++)
	sum += lambda[i];
      for (int i = 0; i < dim+1; i++)
	lambda[i] *= 1.0/sum;

      ElementFunctionDOFVec<T> elFct(&vec);
      elFct.setElInfo(elInfo);
      value = elFct(lambda);
    }
    return value;
  }

} // end namespace experimental
