#include "AMDiS.h"
#include "NavierStokesCahnHilliard.h"
//#include "NavierStokes_TH_MultiPhase.h"
//#include "CahnHilliardNavierStokes_.h"
#include "SignedDistFunctors.h"
#include "PhaseFieldConvert.h"

#include "boost/date_time/posix_time/posix_time.hpp"

using namespace AMDiS;
using namespace boost::posix_time;



class MyNavierStokesCahnHilliard : public NavierStokesCahnHilliard
{
public:
  MyNavierStokesCahnHilliard(std::string name) : NavierStokesCahnHilliard(name)
  { }
  
  void fillBoundaryConditions()
  { FUNCNAME("MyNavierStokes::fillBoundaryConditions()");
    
    DOFVector<double> *zeroDOF = new DOFVector<double>(getFeSpace(0), "zero");
    zeroDOF->set(0.0);
    size_t dow = Global::getGeo(WORLD);

    /// at rigid wall: no-slip boundary condition  
    // oben-unten
    getProblem()->addDirichletBC(2, 0, 0, zeroDOF);
    getProblem()->addDirichletBC(2, 1, 1, zeroDOF);
    // links-rechts
    getProblem()->addDirichletBC(1, 0, 0, zeroDOF);
    //getProblem()->addDirichletBC(1, 1, 1, zeroDOF); 
  }

};



int main(int argc, char** argv)
{ FUNCNAME("main");

  AMDiS::init(argc, argv);
  
  #ifdef HAVE_PARALLEL_DOMAIN_AMDIS
      CreatorMap<OEMSolver>::addCreator("petsc-nsch", new PetscSolverNSCH::Creator);
  #endif
  
    MyNavierStokesCahnHilliard prob("nsch");
  
  // Adapt-Infos
  AdaptInfo adaptInfo("adapt", prob.getNumComponents());
  prob.initialize(INIT_ALL);
  prob.initData();
  prob.initTimeInterface();
  
  AdaptInstationary adaptInstat("adapt", prob, adaptInfo, prob, adaptInfo);

  ptime start_time = microsec_clock::local_time();
  int error_code = adaptInstat.adapt(); 
  time_duration td = microsec_clock::local_time()-start_time;

  MSG("elapsed time= %d sec\n", td.total_seconds());

  AMDiS::finalize();

  return error_code;
};
