/** \file navierStokes.h */

#ifndef NAVIER_STOKES_H
#define NAVIER_STOKES_H

#include "AMDiS.h"
#include "GeometryTools.h"

struct InflowBC : AbstractFunction<double, WorldVector<double> >
{
  InflowBC(double H_=4.1, double Um_=1.5) : H(H_), Um(Um_) {}
  double operator()(const WorldVector<double> &x) const {
    double vel = 4.0 * Um * x[1] * (H - x[1]) / sqr(H);
    return (*time < 2.0 ? vel*(1.0 - cos(m_pi * (*time)/2.0))/2.0 : vel);
  }

  void setTimePtr(double* time_)
  {
    time = time_;
  }
  
protected:
  double H;
  double Um;
  double* time;
};


struct MinWrapper : AbstractFunction<double, WorldVector<double> >
{
  MinWrapper(AbstractFunction<double, WorldVector<double> >* dist1_, AbstractFunction<double, WorldVector<double> >* dist2_)
  : dist1(dist1_), dist2(dist2_) {}

  double operator()(const WorldVector<double>& x) const
  {
    return std::min((*dist1)(x), (*dist2)(x));
  }
private:
  AbstractFunction<double, WorldVector<double> >* dist1;
  AbstractFunction<double, WorldVector<double> >* dist2;
};

#endif
